//----------------------------------------------------------------------
//
// Module Name:  amtu-ia64.c
//
// Include File:  none
//
// Description:   Code for Abstract Machine Test i386 Privilege test.
//
// Notes:  This module performs the machine specific privilege tests
// 		to ensure that the underlying hardware is still enforcing
// 		the appropriate control mechanisms.
// -----------------------------------------------------------------
// LANGUAGE:     C
//
// (C) Copyright International Businesses Machine Corp. 2003
// Licensed under the Common Public License v. 1.0
// -----------------------------------------------------------------
//
// Change Activity:
// DATE     PGMR      COMMENTS
// -------- --------- ----------------------
// 2/05/03  J.Young   Add new X86-64 instructions
// 7/20/03  EJR       Added prolog, comments
// 8/19/03  EJR       Version # on CPL + comment stanzas for functions
// 8/25/03  K.Simon   Added NO_TAG to AUDIT_LOG
// 8/26/03  K.Simon   Added printf to display test name
// 10/17/03 K.Simon   Removed NO_TAG
// 7/15/04  mra       Converted file to be ia64 specific
// 5/27/05  S. Grubb  Update to use libaudit
//----------------------------------------------------------------------

#include "config.h"
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <syscall.h>
#include <errno.h>
#include <string.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <syslog.h>
#include "amtu.h"

#if defined(HAVE_IA64)

/************************************************************************/
/*                                                                      */
/* FUNCTION: catchfault                                                 */
/*                                                                      */
/* PURPOSE: Signal handler to catch the segmentation violation which is */
/*          expected when trying to execute privileged instructions     */
/*          without privilege.                                          */
/*                                                                      */
/************************************************************************/
void catchfault(int sig)
{
	if (debug) {
		printf("caught the fault %d\n", sig);
	}
	exit(0);
}


/************************************************************************/
/*                                                                      */
/* FUNCTION: amtu_priv                                                  */
/*                                                                      */
/* PURPOSE: Execute privileged instructions to ensure that they cannot  */
/*	    legitimately be run in user mode.                           */
/*                                                                      */
/************************************************************************/
int amtu_priv(int argc, char *argv[])
{
	struct sigaction sig;
	pid_t pid, wpid;
	int stat;
                                                                                
	printf("Executing Supervisor Mode Instructions Test...\n");
                                                                                
	/* Set up signal handler */
	sig.sa_handler = catchfault;
	sig.sa_flags = 0;
	sigemptyset(&sig.sa_mask);
	sigaction(SIGSEGV, &sig, NULL);
	sigaction(SIGILL, &sig, NULL);
	sigaction(SIGIOT, &sig, NULL);
	sigaction(SIGIO, &sig, NULL);
	sigaction(SIGINT, &sig, NULL);
	sigaction(SIGABRT, &sig, NULL);
	sigaction(SIGTERM, &sig, NULL);
	sigaction(SIGQUIT, &sig, NULL);
	sigaction(SIGBUS, &sig, NULL);
                                                                                
	/* Each assembly directive should seg fault since they are */
	/* privileged instructions.                                */


        /*---------------------------------------------------------*/
        /* Test One                                                */
        /*---------------------------------------------------------*/

        pid = fork();
        if (pid == 0) {
                if (debug) {
                        printf("RSM test: ");
                }
                asm volatile ("RSM 1");
                exit(-1);
        } else if (pid == -1) {
                /* error condition */
                fprintf(stderr, "Privilege Separation Test FAILED (RSM)!\n");
#ifdef HAVE_LIBLAUS
                LAUS_LOG(("amtu failed privilege separation on RSM"))
#else
                AUDIT_LOG("amtu failed privilege separation on RSM", 0)
#endif
                exit(-1);
        }
        /* parent */
        wpid = wait(&stat);
        if (!(WIFEXITED(stat) && (WEXITSTATUS(stat) == 0))) {
                fprintf(stderr, "Privilege Separation Test FAILED on RSM!\n");
#ifdef HAVE_LIBLAUS
                LAUS_LOG(("amtu failed privilege separation on RSM"))
#else
                AUDIT_LOG("amtu failed privilege separation on RSM", 0)
#endif
                return(-1);
        }


        /*---------------------------------------------------------*/
        /* Test Two                                                */
        /*---------------------------------------------------------*/

        pid = fork();
        if (pid == 0) {
                if (debug) {
                        printf("SSM test: ");
                }
                asm volatile ("SSM 0");
                exit(-1);
        } else if (pid == -1) {
                /* error condition */
                fprintf(stderr, "Privilege Separation Test FAILED (SSM)!\n");
#ifdef HAVE_LIBLAUS
                LAUS_LOG(("amtu failed privilege separation on SSM"))
#else
                AUDIT_LOG("amtu failed privilege separation on SSM", 0)
#endif
                exit(-1);
        }
        /* parent */
        wpid = wait(&stat);
        if (!(WIFEXITED(stat) && (WEXITSTATUS(stat) == 0))) {
                fprintf(stderr, "Privilege Separation Test FAILED on SSM!\n");
#ifdef HAVE_LIBLAUS
                LAUS_LOG(("amtu failed privilege separation on SSM"))
#else
                AUDIT_LOG("amtu failed privilege separation on SSM", 0)
#endif
                return(-1);
        }

        /*---------------------------------------------------------*/
        /* Test Three                                              */
        /*---------------------------------------------------------*/

        pid = fork();
        if (pid == 0) {
                if (debug) {
                        printf("RFI test: ");
                }
                asm volatile ("RFI");
                exit(-1);
        } else if (pid == -1) {
                /* error condition */
                fprintf(stderr, "Privilege Separation Test FAILED (RFI)!\n");
#ifdef HAVE_LIBLAUS
                LAUS_LOG(("amtu failed privilege separation on RFI"))
#else
                AUDIT_LOG("amtu failed privilege separation on RFI", 0)
#endif
                exit(-1);
        }
        /* parent */
        wpid = wait(&stat);
        if (!(WIFEXITED(stat) && (WEXITSTATUS(stat) == 0))) {
                fprintf(stderr, "Privilege Separation Test FAILED on RFI!\n");
#ifdef HAVE_LIBLAUS
                LAUS_LOG(("amtu failed privilege separation on RFI"))
#else
                AUDIT_LOG("amtu failed privilege separation on RFI", 0)
#endif
                return(-1);
        }

#ifdef HAVE_LIBLAUS
	LAUS_LOG(("amtu - Privileged Instruction Test succeeded"))
#else
	AUDIT_LOG("amtu - Privileged Instruction Test succeeded", 1)
#endif
	printf("Privileged Instruction Test SUCCESS!\n");
	return(0);
}
#endif
